import 'dart:io';
import 'dart:async';
import 'dart:convert';
import 'package:flutter/services.dart';
import 'package:image_picker/image_picker.dart';
import 'models/image_picker_response.dart';

class ScanbotImagePickerFlutter {
  static const MethodChannel _channel =
      MethodChannel('scanbot_image_picker_flutter');

  /// pick images from gallery and get the uris list
  static Future<Uri> pickImageAsync() async {
    var uris = await _pickImagesAsync(1);
    if (uris.isNotEmpty) {
      return uris.first;
    } else {
      return Uri();
    }
  }

  /// pick images from gallery and get the uris list
  static Future<List<Uri>> pickImagesAsync() async {
    return await _pickImagesAsync(0);
  }

  /// Get the platform version from native side.
  static Future<String?> get platformVersion async {
    final String? version = await _channel.invokeMethod('getPlatformVersion');
    return version;
  }

  /// if version is less than iOS 14
  /// for Android return false always
  /// true: version is less than 14
  /// false: version is 14 or above
  static Future<bool> isVersionLessThanIOSFourteen() async {
    var isVersionLess = false;
    try {
      var result = await _channel.invokeMethod('versionLessThanIOSFourteen');
      if (result.toString() == "true") {
        isVersionLess = true;
      }
    } catch (e) {
      isVersionLess = false;
    }
    return isVersionLess;
  }

  /// pick images from gallery and get the uris list
  static Future<List<Uri>> _pickImagesAsync(int limit) async {
    var uris = List<Uri>.empty(growable: true);
    if (Platform.isIOS && await isVersionLessThanIOSFourteen()) {
      uris = await _pickerImagesFromNativeImagePicker(limit);
    } else {
      uris = await _pickImagesFromFlutterImagePicker(limit);
    }
    return uris;
  }

  /// pick images from the common flutter image picker.
  static Future<List<Uri>> _pickImagesFromFlutterImagePicker(int limit) async {
    var uris = List<Uri>.empty(growable: true);
    if (limit == 1) {
      // Single image selection
      var image = await ImagePicker().pickImage(
          source: ImageSource.gallery,
          maxWidth: double.infinity,
          maxHeight: double.infinity,
          imageQuality: 75);
      uris.add(Uri.file(image?.path ?? ""));
      return uris;
    }
    // Multiple images selection
    var images = await ImagePicker().pickMultiImage(
      maxWidth: double.infinity,
      maxHeight: double.infinity,
      imageQuality: 75,
    );
    if (images == null || images.isEmpty) {
      return uris;
    } else {
      for (var image in images) {
        uris.add(Uri.file(image.path));
      }
    }
    return uris;
  }

  /// pick images from the common flutter image picker.
  static Future<List<Uri>> _pickerImagesFromNativeImagePicker(int limit) async {
    var uris = List<Uri>.empty(growable: true);
    var result = await _pickImagesFromFlutterChannelAsync(limit);
    if (result?.uris?.isNotEmpty ?? false) {
      for (var path in result?.uris ?? List<String>.empty(growable: false)) {
        uris.add(Uri.file(path));
      }
    }
    return uris;
  }

  /// Pick photos from the library using native code through method channel.
  /// This channel is specifically used only for iOS version below 14.
  static Future<ImagePickerResponse?> _pickImagesFromFlutterChannelAsync(
      int limit) async {
    try {
      var result = await _channel
          .invokeMethod('pickImagesFromPhotosApp', {"limit": limit});
      var resultData = ImagePickerResponse.fromJson(jsonDecode(result));
      return resultData;
    } catch (e) {
      return null;
    }
  }
}
